
open Gamma.Lanczos

(* 分数階微分 L1 アルゴリズム *)
let l1_frac q x coeff j =
	let len = j + 2 and n = j + 1 in
	let x_rev = Array.sub x 0 len |> Array.to_list |> List.rev |> Array.of_list in
	let rl_diff arr l =
		let delta_lq = (float (l+1)) ** (1. -. q) -. (float l) ** (1. -. q) in
		(arr.(l) -. arr.(l+1)) *. delta_lq
	in

	let x_sum = Array.init (len-1) (rl_diff x_rev) |> Array.fold_left (+.) 0. in
	coeff *. (((1. -. q) *. x_rev.(n) /. ((float n) ** q)) +. x_sum)

(* L1アルゴリズムのラッパ *)
let frac_diff q x h =
	Array.init ((Array.length x)-1) (l1_frac q x ((h ** (-.q)) /. gamma (2. -. q))) 


let () =
	let pi = 4. *. atan 1. in
	let x = Array.init 301 (fun t -> sin (2. *. pi *. (float t) /. 100.)) in
	let dq_f = frac_diff 0.5 x 0.01 in
	let print_value t y =
		print_endline ((string_of_float ((float t) *. 0.01)) ^ " " ^ (string_of_float y))
	in
	Array.iteri print_value dq_f