
let pi = 4. *. atan 1.

let ( |> ) x f = f x

(* 複素数の計算 *)
let ( +@ ) z1 z2 = Complex.add z1 z2
let ( -@ ) z1 z2 = Complex.sub z1 z2
let ( *@ ) z1 z2 = Complex.mul z1 z2

(* ビット反転アルゴリズム *)
let mod_2p n p = (n asr p) mod 2
let bit_reversal_index n p =
  Array.init p (fun k -> mod_2p n k) |> Array.to_list |> List.rev |> List.mapi (fun k b -> if b = 1 then (1 lsl k) else 0) |> List.fold_left (+) 0

let bit_reversal p x = Array.init (Array.length x) (fun k -> x.(bit_reversal_index k p)) 

(* 回転子 *)
let w p n =
  let t = -2. *. pi *. (float n) /. (float (1 lsl p)) in
  { Complex.re = cos t; im = sin t }

let wp p n =
  let t = 2. *. pi *. (float n) /. (float (1 lsl p)) in
  { Complex.re = cos t; im = sin t }
    
    
let butterfly n m w x =
  let xn = x.(n) in
  let xm = x.(m) in

  x.(n) <- xn +@ (w *@ xm);
  x.(m) <- xn -@ (w *@ xm);
  ()


let e4 k = Array.init 4 (fun n -> wp 2 (n*k))
let e8 k = Array.init 8 (fun n -> wp 3 (n*k))
let e16 k = Array.init 16 (fun n -> wp 4 (n*k))
let e p k = Array.init (1 lsl p) (fun n -> wp p (n*k)) 

let fft m x =
  let y = bit_reversal m x in
  let w_arr = Array.init (1 lsl m) (fun n -> w m n) in
  let w_val p n = w_arr.( (n lsl (m-p)) ) in
  
  for p = 0 to m-1 do
    let ns = (1 lsl (m-p-1)) - 1 in
    for n = 0 to ns do
      let rs = (1 lsl p) - 1 in
      for r = 0 to rs do
	let ha = (1 lsl (p+1)) in
	let qu = (1 lsl p) in
	butterfly (ha*n+r) (ha*n+qu+r) (w_val (p+1) r) y;
      done;
    done;
  done;

  y
    
