open Int32

(* *)
let le_int32 b0 b1 b2 b3 = 
  logor (of_int b0) (logor (shift_left (of_int b1) 8) (logor (shift_left (of_int b2) 16) (shift_left (of_int b3) 24)))

(* *)
let read_int32 ch =
  let data0 = input_byte ch in
  let data1 = input_byte ch in
  let data2 = input_byte ch in
  let data3 = input_byte ch in
  le_int32 data0 data1 data2 data3

let le_uint16 b0 b1 = b0 lor (b1 lsl 8)
let le_int16 b0 b1 =
  let uval = le_uint16 b0 b1 in
  if uval < 0x8000 then uval else (-32768 + (uval land 0x7fff))

let read_int16 ch =
  let data0 = input_byte ch in
  let data1 = input_byte ch in
  le_int16 data0 data1

type wave_header = {
    wh_riff : Int32.t;
    wh_file_size : Int32.t;
    wh_wave : Int32.t;
    wh_fmt : Int32.t;
    wh_fmt_size : Int32.t;
    wh_fmttag_and_channels : Int32.t;
    wh_samples_per_sec : Int32.t;
    wh_avg_byte_per_sec : Int32.t;
    wh_align_and_adres : Int32.t;
    wh_data : Int32.t;
    wh_data_size : Int32.t
  }

let read_wav_header ch =
  let r_riff = read_int32 ch in
  let r_file_size = read_int32 ch in
  let r_wave = read_int32 ch in
  let r_fmt = read_int32 ch in
  let r_fmt_size = read_int32 ch in
  let r_fmttag_and_channels = read_int32 ch in
  let r_samples_per_sec = read_int32 ch in
  let r_avg_byte_per_sec = read_int32 ch in
  let r_align_and_adres = read_int32 ch in
  let r_data = read_int32 ch in
  let r_data_size = read_int32 ch in
  {
    wh_riff = r_riff;
    wh_file_size = r_file_size;
    wh_wave = r_wave;
    wh_fmt = r_fmt;
    wh_fmt_size = r_fmt_size;
    wh_fmttag_and_channels = r_fmttag_and_channels;
    wh_samples_per_sec = r_samples_per_sec;
    wh_avg_byte_per_sec = r_avg_byte_per_sec;
    wh_align_and_adres = r_align_and_adres;
    wh_data = r_data;
    wh_data_size = r_data_size
  }

let float_of_signal ch n =
  let x = read_int16 ch in
  (* print_endline (string_of_int x); *)
  (* print_endline ((string_of_int n) ^ "," ^ (string_of_int x)); *)
  float_of_int x

let complex_of_signal ch n =
  { Complex.re = float_of_signal ch n; Complex.im = 0. }

let trace_float_array arr = Array.iter (fun z -> print_endline (string_of_float (Complex.norm z))) arr

let analyze_signal ch =
  let l_size = 32768 in
  let x = Array.init l_size (complex_of_signal ch) in
  let y = Fft2.fft 15 x in
  trace_float_array y


let rec dump_waveform ch max cur =
  print_endline (string_of_int (read_int16 ch));
  let next = cur + 1 in
  if next >= max then ()
  else dump_waveform ch max next

let () =
  let fname = Sys.argv.(1) in
  let ch_wav = open_in_bin fname in
  ignore (read_wav_header ch_wav);
  analyze_signal ch_wav;
  ignore (close_in ch_wav)

