
Require Import Arith.
Require Import List.
Open Scope list_scope.


Record material := { weight : nat; value : nat }.

Definition material_add (m1 m2 : material) : material :=
  Build_material (weight m1 + weight m2) (value m1 + value m2).

Lemma material_add_comm : forall (m1 m2 : material), material_add m1 m2 = material_add m2 m1.
Proof.
  intros.
  unfold material_add.
  SearchAbout "comm". (* 自然数について使えそうな定理を探す *)
  (* 各成分について交換法則が成り立つことを証明する *)
  assert (weight m1 + weight m2 = weight m2 + weight m1) as Hw.
  apply NPeano.Nat.add_comm.
  assert (value m1 + value m2 = value m2 + value m1) as Hv.
  apply NPeano.Nat.add_comm.
  (* あとは rewrite -> reflexivity の流れ *)
  rewrite Hw.
  rewrite Hv.
  reflexivity.
Qed.

Fixpoint material_add_list (ml : list material) : material :=
  match ml with
  | nil => Build_material 0 0
  | x :: xs => material_add x (material_add_list xs)
  end.

Compute material_add_list ((Build_material 5 5) :: (Build_material 2 4) :: (Build_material 3 7) :: nil).

Lemma weight_add : forall (m1 m2 : material), weight (material_add m1 m2) = weight m1 + weight m2.
Proof.
  intros.
  unfold material_add.
  simpl.
  reflexivity.
Qed.

Lemma value_add : forall (m1 m2 : material), value (material_add m1 m2) = value m1 + value m2.
Proof.
  intros.
  unfold material_add.
  simpl.
  reflexivity.
Qed.


Lemma same_weights : forall (m m1 m2 : material), weight m1 = weight m2 -> weight (material_add m1 m) = weight (material_add m2 m).
Proof.
  unfold material_add.
  intros.
  simpl.
  SearchAbout "add_cancel_r".
  apply NPeano.Nat.add_cancel_r.
  exact H.
Qed.

Lemma plus_gt_compat_r : forall (n m p : nat), n > m -> n + p > m + p.
Proof.
  intros.
  assert (n + p = p + n) as Hnp.
  apply plus_comm.
  assert (m + p = p + m) as Hmp.
  apply plus_comm.
  rewrite Hnp.
  rewrite Hmp.
  SearchAbout "plus_gt".
  apply plus_gt_compat_l.
  exact H.
Qed.
  

Lemma higher_values : forall (m m1 m2 : material), value m1 > value m2 -> value (material_add m1 m) > value (material_add m2 m).
Proof.
  unfold material_add.
  intros.
  simpl.
  apply plus_gt_compat_r.
  exact H.
Qed.

Definition higher_value_material (m1 m2 : material) : Prop :=
  weight m1 = weight m2 /\ value m1 > value m2.

Lemma higher_material_add : forall (m1 m2 m : material), higher_value_material m1 m2 -> higher_value_material (material_add m1 m) (material_add m2 m).
Proof.
  intros.
  unfold higher_value_material in H.
  unfold higher_value_material.
  assert (weight m1 = weight m2) as Hw.
  apply H.
  assert (value m1 > value m2) as Hv.
  apply H.
  split.
  apply same_weights.
  exact Hw.
  apply higher_values.
  exact Hv.
Qed.