
let pi = 4. *. atan 1.
let circle_point (x, y) r n_max idx =
  let theta = 2. *. pi *. (float idx) /. (float n_max) in
  (x +. r *. cos theta, y +. r *. sin theta)

let circle_points (x, y) r = Array.init 31 (circle_point (x, y) r 31)

let ( +! ) (u1,u2) (v1,v2) =
  (u1 +. v1, u2 +. v2)

let ( *< ) c (u1, u2) =
  (c *. u1, c *. u2)
    
let runge_kutta_next (y, v) df h =
  let (ky1, kv1) = df (y, v) in
  let (ky2, kv2) = df ((y +! ((0.5*.h) *< ky1)), (v +! ((0.5*.h) *< kv1))) in
  let (ky3, kv3) = df ((y +! ((0.5*.h) *< ky2)), (v +! ((0.5*.h) *< kv2))) in
  let (ky4, kv4) = df ((y +! (h *< ky3)), (v +! (h *< kv3))) in
  (y +! ((h/.6.) *< (ky1 +! (2. *< ky2) +! (2. *< ky3) +! ky4)), v +! ((h/.6.) *< (kv1 +! (2. *< kv2) +! (2. *< kv3) +! kv4)))

let mb = 100.
let mu = 5.
(* ボイスコイルにより k0 の値を調整 *)
let k0 = 250000.
(* let k0 = 50000. *)
let g = 1.

let n = ref 0

let df ((yb,yu), (vb,vu)) =
  let ln = 7. in (* 自然長 *)
  let dy = if vb > vu && vb > 0. && yu -. yb < 5. then ln -. (yu -. yb) else 0. in
  let au = if yu < 50. then k0 /. mu *. dy -. g else 0. in
  ((vb,vu), (-.(k0 /. mb) *. dy -. g, au))


let yb = ref 40.
let vb = ref (0.)

let yu = ref 45.
let vu = ref (0.)

let xb = ref 0.
let xu = ref 0.


let () =
  ignore (Glut.init Sys.argv);
  Glut.initDisplayMode ~double_buffer:true ();
  Glut.initWindowSize ~w:800 ~h:600;
  ignore (Glut.createWindow ~title:"LablGL Triple Demo");
  GlClear.color (0.0, 0.0, 0.2);
  GlDraw.shade_model `smooth;
	
  let render () =
    GlMat.mode `projection;
    GlMat.load_identity ();
    GluMat.ortho2d ~x:(-40., 40.) ~y:(0., 60.);
    GlMat.mode `modelview;
    GlMat.load_identity ();
    GlClear.clear [ `color ];
    let ((yb_new, yu_new), (vb_new, vu_new)) = runge_kutta_next ((!yb, !yu), (!vb, !vu)) df 0.002 in
    yb := yb_new; yu := yu_new;
    vb := if !yb <= 5. then -.0.9*.vb_new else vb_new; vu := vu_new;
    xb := 0.; xu := 0.;

    GlDraw.begins `quads;
    GlDraw.vertex2 (!xb-.5., !yb-.5.);
    GlDraw.vertex2 (!xb-.3., !yb-.5.);
    GlDraw.vertex2 (!xb-.3., !yb-.2.);
    GlDraw.vertex2 (!xb-.5., !yb-.2.);
    GlDraw.ends ();

    GlDraw.begins `quads;
    GlDraw.vertex2 (!xb+.5., !yb-.5.);
    GlDraw.vertex2 (!xb+.3., !yb-.5.);
    GlDraw.vertex2 (!xb+.3., !yb-.2.);
    GlDraw.vertex2 (!xb+.5., !yb-.2.);
    GlDraw.ends ();

    GlDraw.begins `quads;
    GlDraw.vertex2 (!xb+.5., !yb-.2.);
    GlDraw.vertex2 (!xb-.5., !yb-.2.);
    GlDraw.vertex2 (!xb-.5., !yb);
    GlDraw.vertex2 (!xb+.5., !yb);
    GlDraw.ends ();

    
    GlDraw.begins `polygon;
    Array.iter GlDraw.vertex2 (circle_points (!xu, !yu) 0.5);
    GlDraw.ends ();
    n := !n + 1;
    Gl.flush ();
    Glut.swapBuffers ()
  in
  Glut.displayFunc ~cb:render;
  Glut.idleFunc ~cb:(Some Glut.postRedisplay);
  (* Escキーで終了 *)
  Glut.keyboardFunc ~cb:(fun ~key ~x ~y -> if key=27 then exit 0);
  Glut.mainLoop ()
