
(* 遅延評価による Euler 法の計算  *)
open Lazy

exception Empty_stream

type 'a stream = Nils | Cons of 'a * 'a stream Lazy.t

let eom_euler h a (xp, vp) =
  let vn = vp +. h *.  a (xp, vp) in
  let xn = xp +. h *. vn in
  (xn, vn)
 
let rec lazy_eom_euler h a (x0, v0) = Cons ((x0, v0), lazy (lazy_eom_euler h a (eom_euler h a (x0, v0))))

let rec stream_ref s n =
  match s with
  | Nils -> raise Empty_stream
  | Cons (x, _) when n = 1 -> x
  | Cons (_, tail) -> stream_ref (force tail) (n-1)

let tpi = 8. *. atan 1.
let a (x, v) = -.(tpi*.tpi) *. x
let s = lazy_eom_euler 0.0001 a (0., tpi)
let print_vector_fst (x, v) = print_float x
let rec dump_stream n idx =
  print_vector_fst (stream_ref s (idx+1));
  print_endline "";
  if (idx + 1) > n then ()
  else dump_stream n (idx+1)

let () = dump_stream 30000 0
