
(*
 ミックスオイラー法に関する利用上の注意 : 
 ミックスオイラー法を用いた数値解析装置及び数値解析プログラムは,
 北陸電力株式会社が日本および米国内で特許出願中であり, 
 商用目的等での使用を希望される場合は, 事前に北陸電力株式会社までご連絡下さい. 
   http://www.rikuden.co.jp/products/shouhin19.html
 *)
open Lazy
exception Empty_stream

(* 遅延ストリーム *)
type 'a stream = Nils | Cons of 'a * 'a stream Lazy.t

(* mixed euler 法で次のステップの値を計算 *)
let mixed_euler_next h a (x_pre, v_pre) =
  let v_cur = v_pre +. h *. a (x_pre, v_pre) in
  let x_cur = x_pre +. h *. v_cur in
  print_float x_cur; print_endline "";
  (x_cur, v_cur)

(* mixed-euler 法による遅延ストリーム定義 *)
let rec lazy_mixed_euler h a (x0, v0) = Cons ((x0, v0), lazy (lazy_mixed_euler h a (mixed_euler_next h a (x0, v0))))

(* *)
let rec stream_ref s n =
  match s with
  | Nils -> raise Empty_stream
  | Cons (x, _) when n = 1 -> x
  | Cons (_, tail) -> stream_ref (force tail) (n-1)

let tpi = 8. *. atan 1.
let a (x,v) = -.(tpi*.tpi) *. x
let s = lazy_mixed_euler 0.0001 a (0., 1.)
let () =
  let (x, v) = stream_ref s 30000 in
  ()
