
open Lacaml.D

(*  *)
let row_add u v =
  Array.init (Array.length u) (fun n -> u.(n) +. v.(n))
    
let row_sub u v =
  Array.init (Array.length u) (fun n -> u.(n) -. v.(n))

let row_time c v =
  Array.init (Array.length v) (fun n -> c *. v.(n))

let print_vector v =
  Array.iter (fun w -> print_string (string_of_float w)) v
    

(* 行列の演算 *)
let ( +! ) a b =
  Array.init (Array.length a) (fun n -> row_add a.(n) b.(n))
let ( -! ) a b =
  Array.init (Array.length a) (fun n -> row_sub a.(n) b.(n))
let ( *< ) c a =
  Array.init (Array.length a) (fun n -> row_time c a.(n))

(* 積 : Lacaml 使用 *)
let ( *! ) a b =
  let a_lm = lacpy (Mat.of_array a) in
  let b_lm = lacpy (Mat.of_array b) in
  Mat.to_array (gemm a_lm b_lm)

let kronecker i j = if i = j then 1. else 0.
let id_m n = Array.init n (fun i -> Array.init n (kronecker i)) 
    
let rec pow a r n =
  if n <= 0 then a
  else pow (a *! r) r (n-1)  

(* 行列 a の n 乗 *)
let ( *^ ) a n =
  pow ([| [|1.;0.|]; [|0.;1.|] |]) a n

let print_matrix arr2 =
  Array.iter (fun r -> print_vector r) arr2;
  print_endline ""

let inverse a b =
  let a_lm = lacpy (Mat.of_array a) in
  let b_lm = lacpy (Mat.of_array b) in
  gesv a_lm b_lm;
  Mat.to_array b_lm

(* Pade 11 法 *)
let exp_m h a =
  let r = Array.length a in
  inverse (id_m r -! ((0.5 *. h) *< a)) (id_m r +! ((0.5 *. h) *< a))

let () =
  let h = 0.01 in
  let a = [| [|0.; -1.|]; [|1.; 0.|] |] in
  let b = [| [|cos h; -.sin h|]; [|sin h; cos h|] |] in
  let c = exp_m h a in

  print_float b.(0).(0);
  print_string " ";
  print_float b.(0).(1);
  print_endline "";
  print_float b.(1).(0);
  print_string " ";
  print_float b.(1).(1);
  print_endline "";
  
  print_float c.(0).(0);
  print_string " ";
  print_float c.(0).(1);
  print_endline "";
  print_float c.(1).(0);
  print_string " ";
  print_float c.(1).(1);
  print_endline ""
